
#ifndef _G_GPU_HALSPEC_NVOC_H_
#define _G_GPU_HALSPEC_NVOC_H_

// Version of generated metadata structures
#ifdef NVOC_METADATA_VERSION
#undef NVOC_METADATA_VERSION
#endif
#define NVOC_METADATA_VERSION 2

#include "nvoc/runtime.h"
#include "nvoc/rtti.h"

#ifdef __cplusplus
extern "C" {
#endif

/*
 * SPDX-FileCopyrightText: Copyright (c) 2021-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 * SPDX-License-Identifier: MIT
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#pragma once
#include "g_gpu_halspec_nvoc.h"

#ifndef GPU_HALSPEC_H
#define GPU_HALSPEC_H

#include "g_chips2halspec.h"        // NVOC halspec, generated by rmconfig.pl


// Private field names are wrapped in PRIVATE_FIELD, which does nothing for
// the matching C source file, but causes diagnostics to be issued if another
// source file references the field.
#ifdef NVOC_GPU_HALSPEC_H_PRIVATE_ACCESS_ALLOWED
#define PRIVATE_FIELD(x) x
#else
#define PRIVATE_FIELD(x) NVOC_PRIVATE_FIELD(x)
#endif


// Metadata with per-class RTTI
struct NVOC_METADATA__RmHalspecOwner;


struct RmHalspecOwner {

    // Metadata starts with RTTI structure.
    union {
         const struct NVOC_METADATA__RmHalspecOwner *__nvoc_metadata_ptr;
         const struct NVOC_RTTI *__nvoc_rtti;
    };

    // Ancestor object pointers for `staticCast` feature
    struct RmHalspecOwner *__nvoc_pbase_RmHalspecOwner;    // rmhalspecowner

    // Data members
    struct RmVariantHal rmVariantHal;
    struct DispIpHal dispIpHal;
};


// Metadata with per-class RTTI
struct NVOC_METADATA__RmHalspecOwner {
    const struct NVOC_RTTI rtti;
};

#ifndef __nvoc_class_id_RmHalspecOwner
#define __nvoc_class_id_RmHalspecOwner 0x34a6d6u
typedef struct RmHalspecOwner RmHalspecOwner;
#endif /* __nvoc_class_id_RmHalspecOwner */

// Casting support
extern const struct NVOC_CLASS_DEF __nvoc_class_def_RmHalspecOwner;

#define __staticCast_RmHalspecOwner(pThis) \
    ((pThis)->__nvoc_pbase_RmHalspecOwner)

#ifdef __nvoc_gpu_halspec_h_disabled
#define __dynamicCast_RmHalspecOwner(pThis) ((RmHalspecOwner*) NULL)
#else //__nvoc_gpu_halspec_h_disabled
#define __dynamicCast_RmHalspecOwner(pThis) \
    ((RmHalspecOwner*) __nvoc_dynamicCast(staticCast((pThis), Dynamic), classInfo(RmHalspecOwner)))
#endif //__nvoc_gpu_halspec_h_disabled

NV_STATUS __nvoc_objCreateDynamic_RmHalspecOwner(RmHalspecOwner**, Dynamic*, NvU32, va_list);

NV_STATUS __nvoc_objCreate_RmHalspecOwner(RmHalspecOwner**, Dynamic*, NvU32,
        RM_RUNTIME_VARIANT RmVariantHal_rmVariant,
        NvU32 DispIpHal_ipver);
#define __objCreate_RmHalspecOwner(__nvoc_ppNewObj, __nvoc_pParent, __nvoc_createFlags, RmVariantHal_rmVariant, DispIpHal_ipver) \
    __nvoc_objCreate_RmHalspecOwner((__nvoc_ppNewObj), staticCast((__nvoc_pParent), Dynamic), (__nvoc_createFlags), RmVariantHal_rmVariant, DispIpHal_ipver)

// Virtual method declarations and/or inline definitions
// Exported method declarations and/or inline definitions
// HAL method declarations without bodies
// Inline HAL method definitions
// Static dispatch method declarations
// Static inline method definitions
#undef PRIVATE_FIELD



// Private field names are wrapped in PRIVATE_FIELD, which does nothing for
// the matching C source file, but causes diagnostics to be issued if another
// source file references the field.
#ifdef NVOC_GPU_HALSPEC_H_PRIVATE_ACCESS_ALLOWED
#define PRIVATE_FIELD(x) x
#else
#define PRIVATE_FIELD(x) NVOC_PRIVATE_FIELD(x)
#endif


// Metadata with per-class RTTI
struct NVOC_METADATA__GpuHalspecOwner;


struct GpuHalspecOwner {

    // Metadata starts with RTTI structure.
    union {
         const struct NVOC_METADATA__GpuHalspecOwner *__nvoc_metadata_ptr;
         const struct NVOC_RTTI *__nvoc_rtti;
    };

    // Ancestor object pointers for `staticCast` feature
    struct GpuHalspecOwner *__nvoc_pbase_GpuHalspecOwner;    // gpuhalspecowner

    // Data members
    struct ChipHal chipHal;
    struct TegraChipHal tegraChipHal;
};


// Metadata with per-class RTTI
struct NVOC_METADATA__GpuHalspecOwner {
    const struct NVOC_RTTI rtti;
};

#ifndef __nvoc_class_id_GpuHalspecOwner
#define __nvoc_class_id_GpuHalspecOwner 0x74bc71u
typedef struct GpuHalspecOwner GpuHalspecOwner;
#endif /* __nvoc_class_id_GpuHalspecOwner */

// Casting support
extern const struct NVOC_CLASS_DEF __nvoc_class_def_GpuHalspecOwner;

#define __staticCast_GpuHalspecOwner(pThis) \
    ((pThis)->__nvoc_pbase_GpuHalspecOwner)

#ifdef __nvoc_gpu_halspec_h_disabled
#define __dynamicCast_GpuHalspecOwner(pThis) ((GpuHalspecOwner*) NULL)
#else //__nvoc_gpu_halspec_h_disabled
#define __dynamicCast_GpuHalspecOwner(pThis) \
    ((GpuHalspecOwner*) __nvoc_dynamicCast(staticCast((pThis), Dynamic), classInfo(GpuHalspecOwner)))
#endif //__nvoc_gpu_halspec_h_disabled

NV_STATUS __nvoc_objCreateDynamic_GpuHalspecOwner(GpuHalspecOwner**, Dynamic*, NvU32, va_list);

NV_STATUS __nvoc_objCreate_GpuHalspecOwner(GpuHalspecOwner**, Dynamic*, NvU32,
        NvU32 ChipHal_arch, NvU32 ChipHal_impl, NvU32 ChipHal_hidrev,
        TEGRA_CHIP_TYPE TegraChipHal_tegraType);
#define __objCreate_GpuHalspecOwner(__nvoc_ppNewObj, __nvoc_pParent, __nvoc_createFlags, ChipHal_arch, ChipHal_impl, ChipHal_hidrev, TegraChipHal_tegraType) \
    __nvoc_objCreate_GpuHalspecOwner((__nvoc_ppNewObj), staticCast((__nvoc_pParent), Dynamic), (__nvoc_createFlags), ChipHal_arch, ChipHal_impl, ChipHal_hidrev, TegraChipHal_tegraType)

// Virtual method declarations and/or inline definitions
// Exported method declarations and/or inline definitions
// HAL method declarations without bodies
// Inline HAL method definitions
// Static dispatch method declarations
// Static inline method definitions
#undef PRIVATE_FIELD


#endif // GPU_HALSPEC_H

#ifdef __cplusplus
} // extern "C"
#endif

#endif // _G_GPU_HALSPEC_NVOC_H_
